<?PHP
/**
* Load and parse a config file
*
* Load and parse a configuration file with a name=value format.
*
* @package utilities
* @param string		$filename		Path and name of configuration file
* @param array		$permitted		List of permitted keys
* @return array		Array containing config file elements
*/
function loadConfig( $filename, $permitted ) {
	$skipLineRegex = '/(#.*|^\s$)/i';
	$cleanStartRegex = "/^[\'\"]/";
	$cleanEndRegex = "/[\'\"]$/";
	$fptr = fopen( $filename, 'r' );
	if( !$fptr ) {
		return NULL;
	}
	
	while( $line = fgets( $fptr, 1024 ) ) {
		if( preg_match( $skipLineRegex, $line ) ) continue;
		list( $key, $value ) = explode( "=", $line );
		$key = trim( $key );
		$value = trim( $value );
		$value = preg_replace( $cleanStartRegex, "", $value );
		$value = preg_replace( $cleanEndRegex, "", $value );
		foreach( $permitted as $pkey ) {
			if( $key == $pkey && $key && $value ) $config[$key] = $value;
		}
	}
	@fclose( $fptr );
	return $config;
}

$configFile = 'mindmeld.config';
$permittedKeys = array( 'home' );

$config = loadConfig( $configFile, $permittedKeys );
if( !file_exists( "{$config['home']}$configFile" ) ) {
	die( "No configuration at {$config['home']}$configFile" );
} else {
	
	$configFile = "{$config['home']}$configFile";
	$permittedKeys = array( 'home', 'webhome', 'site', 'theme', 'cache',
							'dbType', 'dbName', 'dbHost', 'dbPort',
							'dbUser', 'dbPass', 'debug', 'sqlDebug',
							'adodb', 'adodbCache', 'smarty_home',
							'postnuke_home' );
	
	$config = loadConfig( $configFile, $permittedKeys );
	foreach( $config as $key => $value ) {
		$MM_GLOBALS[$key] = $value;
	}
}

// Set defaults for undefined keys
foreach( array_values( $permittedKeys ) as $key ) {
	
	if ( !key_exists( $key, $MM_GLOBALS ) ) {
		switch( $key ) {
			case 'home':
				$MM_GLOBALS['home'] = dirname( '../' . __FILE__ );
				break;
			case 'webhome':
				$MM_GLOBALS['webhome'] = $MM_GLOBALS['home'] . "acweb";
				break;
			case 'site':
				$MM_GLOBALS['site'] = preg_replace( "/\/$/", "", $GLOBALS['PHP_SELF'] );
				break;
			case 'cache':
				$MM_GLOBALS['cache'] = $MM_GLOBALS['home'] . "cache/";
				break;
			case 'theme':
				$MM_GLOBALS['theme'] = "arsgreen";
				break;
			case 'dbType':
				$MM_GLOBALS['dbType'] = "postgres7";
				break;
			case 'dbName':
				$MM_GLOBALS['dbName'] = "mindmeld";
				break;
			case 'dbHost':
				$MM_GLOBALS['dbHost'] = "localhost";
				break;
			case 'dbPort':
				$MM_GLOBALS['dbPort'] = "";
				break;
			case 'dbUser':
				$MM_GLOBALS['dbUser'] = "sa";
				break;
			case 'dbPass':
				$MM_GLOBALS['dbPass'] = "";
				break;
			case 'debug':
				$MM_GLOBALS['debug'] = -1;
				break;
			case 'sqlDebug':
				$MM_GLOBALS['sqlDebug'] = FALSE;
				break;
			case 'adodb':
				$MM_GLOBALS['adodb'] = $MM_GLOBALS['home'] . "opensource/adodb/";
				break;
			case 'smarty_home':
				$MM_GLOBALS['smarty_home'] = $MM_GLOBALS['home'] . "opensource/smarty/";
				break;
			case 'postnuke_home':
				//$MM_GLOBALS['postnuke_home'] = $MM_GLOBALS['site'] . "opensource/smarty";
				break;
			default:
		}
	}
}


$PHPLOT_HOME = $MM_GLOBALS['home']."opensource/phplot";

$MM_GLOBALS['includes'] = $MM_GLOBALS['home'] . "include/";
$MM_GLOBALS['docroot'] = $_SERVER['DOCUMENT_ROOT'] . $MM_GLOBALS['site'];
$MM_GLOBALS['smarty_templates'] = $MM_GLOBALS['home']."templates";

// Configure cache locations
$MM_GLOBALS['smarty_compiled_templates'] = $MM_GLOBALS['cache']."templates_c";
$MM_GLOBALS['smarty_cache_dir'] = $MM_GLOBALS['cache']."html_cache";
$MM_GLOBALS['sql_cache_dir'] = $MM_GLOBALS['cache']."sql_cache";

define( "SMARTY_DIR", $MM_GLOBALS['smarty_home'] );
?>
