<?PHP
// Copyright (c) 2003 ars Cognita, Inc., all rights reserved
/********************************************************************************
	
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
* Database utilities for the base mindmeld installer.
*
* @author 		$Author: richtl $
* @version 	$Revision: 1388 $
*
* @package		installer
*
* @internal
*/

include 'install_strings.inc';

/**
* Input Security Policy
*/
define( 'VALID_PATH', 'MAXLENGTH=255' );
define( 'VALID_STRING', 'MAXLENGTH=30' );
define( 'VALID_PORTNUM', 'MAXLENGTH=6|MIN=0|MAX=999999' );

/**
 Default mindmeld home directory
 */
define( 'DEF_HOME', '/usr/local/lib/mindmeld' );
define( 'DEF_HOME_LINUX', '/usr/local/lib/mindmeld' );
define( 'DEF_HOME_WINDOWS', '/Program Files/Apache Group/APACHE/htdocs/mindmeld' );
	
/**
* Base installer class
*
* The base class for the mindmeld installer. Provides basic
* and methods common to all installer objects.
*/
class installer_base {
	
	/**
	* @var array	List of checklist requirements
	*/
	var $requirements;
	
	/**
	* @var array	Checklist of finished items
	*/
	var $checklist;
	
	/**
	* @var array	Current error message for items
	*/
	var $messages;
	
	/**
	* @var array	Values for installation items
	*/
	var $installValues;
	
	/**
	* @var array	Values for config items
	*/
	var $configValues;
	
	/**
	* @var string	Last good state
	*/
	var $lastState;
	
	/**
	* Specify the install step based on what we have
	*
	* Determine and return the next installation state
	* based upon finished checklist items.
	*
	* @return integer	State to execute
	*/
	function getState() {
		
		// Look through the requirements and checklist for the
		// first unfullfilled level.
		
		// How to do it: Make a temporary copy of the requirements array,
		// removing every item that is marked as "done" on the checklist.
		// Finally, return the lowest remaining state value from the list.
		
		$reqs = $this->requirements;
				
		foreach( $this->checklist as $item => $checked ) {
			if( $checked and isset( $reqs[$item] ) ) unset( $reqs[$item] );
		}
		asort( $reqs );
		$currState = reset( $reqs );
		
		return $currState;
	}
	
	/**
	* Write a config file
	*
	* @param string	$item	Checklist name for this config write
	* @param string	$path	Path to directory
	* @param array	$values	Array of items and values
	* @return boolean	NULL if successful, else error message
	*/
	function writeConfig( $item, $path, $values ) {
		
		// Make sure path ends in a slash
		$filename = preg_replace( '/^(.*)\/?$/', '\1/mindmeld.config', $path );
		
		// Create the file handle
		if( file_exists( $filename ) and $err = $this->inaccessible( $filename, 'WRITE' ) ) {
			// Can't write install config
			return $err;
		}
		
		$fp = fopen( $filename, 'w' );
		
		// Loop through config items and write to file
		foreach( $values as $key => $value ) {
			fputs( $fp, "$key = $value\n" );
		}
		fclose( $fp );

		return NULL;
	}
	
	/**
	* Build HTML for Step 0: Introduction
	*
	* Display an introduction to the installer.
	*
	* @return string	HTML
	*/
	function step0() {
		
		// Check off items on the checklist
		//$this->check( 'begin' );
		
		// Use heredoc to build the HTML
		$istep = 'intro';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";
		$html .= "<SPAN CLASS='page_prompt'>" . $mmiStrings[$istep]['prompt'] . "</SPAN>";
		
		$html .=
			  "<DIV CLASS='form'>\n"
  			. "	<FORM METHOD=POST ACTION={$_SERVER['SCRIPT_NAME']}>\n"
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='submit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Begin></SPAN>\n"
			. "		</DIV>\n"
			. "	</FORM>\n"
			. "</DIV>\n";

		return $html;
	}
	
	/**
	* Build HTML for Step 5: Default Page
	*
	* Collect information for the following items: home, webhome,
	* and site.
	*
	* @return string	HTML
	*/
	function step5() {
		
		// Set web site location
		if( $this->get_install( 'site' ) ) { 
			$site = $this->get_install( 'site' );
		} else {
			$site = preg_replace( '/\/[^\/]+\/?$/', '', dirname( $_SERVER['SCRIPT_NAME'] ) );
		}
		
		// Set web home directory
		if( $this->get_install( 'webhome' ) ) { 
			$webhome = $this->get_install( 'webhome' );
		} else {
			// Set Mindmeld's webhome to the parent of the current directory
			$webhome = preg_replace( '/\/[^\/]+\/?$/', '', getcwd() );
			$webhome = str_replace( '\\\\', '\\', $webhome );
		}
		
		// Set Mindmeld's home (search for it first)
		
		if( $this->get_install( 'home' ) ) { 
			$home = $this->get_install( 'home' );
		} else { 
			
			// Default home by OS type
			switch( preg_replace( '/^WIN.*/', 'Windows', PHP_OS ) ) {
				case "Windows":
					$homeByOS = DEF_HOME_WINDOWS;
					break;
				case "Linux":
					$homeByOS = DEF_HOME_LINUX;
					break;
				default:
					break;
			}
			
			// Default home for insecure installation
			$homeInsecure = preg_replace( '/(?i)\/acweb\/?$/', '', $webhome );
		}
		
		if( !isset( $homeInsecure ) ) $homeInsecure = '';
		if( !isset( $homeByOS ) ) $homeByOS = '';
		if( !isset( $site ) ) $site = '';
		
		// Update messages array
		if( $this->messages ) {
			foreach( $this->messages as $key => $message ) {
				$msg[$key] = '<FONT COLOR=#AA0000>' . $message . '</FONT><BR/>';
			}
		}
			
		// Check off items on the checklist
		$this->check( 'base' );
		
		// Use heredoc to build the HTML
		$istep = 'defaults';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";
		$html .= "<SPAN CLASS='page_prompt'>" . $mmiStrings[$istep]['prompt'] . "</SPAN>";

		$html .=
			  "<DIV CLASS='form'>\n"
			. "	<FORM METHOD=POST ACTION={$_SERVER['SCRIPT_NAME']}>\n"
			. "		<DIV CLASS='fsubmit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></DIV>"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN class='label'>Mindmeld Web Directory</SPAN>\n"
			. "			<SPAN CLASS='formw'><INPUT TYPE=TEXT NAME=webhome VALUE='$webhome' SIZE=40/></SPAN>\n"
			. "			<SPAN CLASS='warn'>{$msg['webhome']}</SPAN>\n"		
			. "			<SPAN CLASS='descr'><P>Verify the <em>filesystem path</em> to Mindmeld's <b>acweb</b> directory. "
			. "				This directory contains all the files that will be accessed by your web server, and is "
			. "				located initially in your main Mindmeld directory.</P>"
			. "				SECURITY TIP: If you plan to use Mindmeld on the open Internet, you should install the main "
			. "				Mindmeld directory in a location that is <em>not</em> accessable by your web server, and "
			. "				then copy only the <b>acweb</b> directory to where it can be served to the web.</P>\n"
			. "			</SPAN>\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='label'>Mindmeld Home Directory</SPAN>"
			. "			<SPAN CLASS='formw' >\n"
			. "				<INPUT TYPE=RADIO NAME=home VALUE='$homeInsecure' CHECKED> $homeInsecure</INPUT><BR/>"
			. "				<INPUT TYPE=RADIO NAME=home VALUE='$homeByOS'> $homeByOS</INPUT><BR/>"
			. "				<INPUT TYPE=RADIO NAME=home VALUE='alternate'> "
			. "					<INPUT TYPE=TEXT NAME=home_alt VALUE='' SIZE=30/>"				
			. "				</INPUT>"
			. "			</SPAN>\n"
			. "			<SPAN CLASS='warn'>{$msg['home']}</SPAN>"
			. "			<SPAN CLASS='descr'><P>Enter the <em>filesystem path</em> to the location where you "
			. "				installed the Mindmeld package. This is the Mindmeld <em>home directory</em>. "
			. "				SECURITY TIP: If you plan to use Mindmeld on the open Internet, you should install the Mindmeld "
			. "				home directory in a location that is <em>not</em> accessable by your web server."
			. "			</SPAN>\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='label'>Mindmeld Web Root</SPAN>\n"
			. "			<SPAN CLASS='formw'><INPUT TYPE=TEXT NAME=site VALUE='$site' SIZE=40/></SPAN>"
			. "			<SPAN CLASS='warn'>{$msg['site']}</SPAN>"
			. "			<SPAN CLASS='descr'><P>Verify the <em>web address</em> you would put into your "
			. "				web browser to browse to the location where the Mindmeld <em>web home</em> is installed. "
			. "			</SPAN\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='submit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></SPAN>"
			. "		</DIV>\n"
			. "	</FORM>"
			. "</DIV>";
	
		return $html;
	}
	
	/**
	* Build HTML for Step 10
	*
	* Validate the following items: introduction, home, webhome,
	* and site. Collect information for over-riding: include, cache,
	* sql_cache, and adodb_path.
	*
	* @return string	HTML
	*/
	function step10() {
		
		$success = TRUE;
		
		// Validate data from the last step
		
		$regex = '/^(.*)\/?$/';
		
		$this->set_install( 'home', preg_replace( '/\/$/', '', $this->get_install( 'home' ) ) );
		$home = $this->get_install( 'home' );
		
		// Validate mindmeld home
		$check = array( $home => 'WRITE' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'home', $err );
			$success = FALSE;
		} else {
			$this->check( 'home' );
		}
		
		// Validate webhome
		$webhome = $this->get_install( 'webhome' );
		$check = array( $webhome => 'WRITE' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'webhome', $err );
			$success = FALSE;
		} else {
			$this->check( 'webhome' );
		}
		
		// Not currently checking site
		$this->check( 'site' );
		
		// If any of the above fail, uncheck introduction and refresh.
		if( !$success ) return NULL;
		
		
		// Initialize the form for the next step
		
		// Set secondary paths
		if( ! $include = $this->get_install( 'include' ) ) {
			$this->set_install( 'include', "$home/include" );
			$sql_cache = $this->get_install( 'include' );
		}
		
		if( ! $cache = $this->get_install( 'cache' ) ) {
			$this->set_install( 'cache', "$home/cache" );
			$sql_cache = $this->get_install( 'cache' );
		}
		
		if( ! $sql_cache = $this->get_install( 'sql_cache' ) ) {
			$this->set_install( 'sql_cache', $this->get_install( 'cache' ) . "/sql_cache" );
			$sql_cache = $this->get_install( 'sql_cache' );
		}
		
		if( ! $adodb_path = $this->get_install( 'adodb_path' ) ) {
			$this->set_install( 'adodb_path', "$home/opensource/adodb" );
			$adodb_path = $this->get_install( 'adodb_path' );
		}
		
		if( $this->messages ) {
			foreach( $this->messages as $key => $message ) {
				$msg[$key] = '<FONT COLOR=#AA0000>' . $message . '</FONT><BR/>';
			}
		}
		
		// Collect data for next step
		// Use heredoc to build the HTML
		$istep = 'directories';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";
		$html .= "<SPAN CLASS='page_prompt'>" . $mmiStrings[$istep]['prompt'] . "</SPAN>";
		
		$html .= 
			  "<DIV CLASS='form'>\n"
			. "	<FORM METHOD=POST ACTION={$_SERVER['SCRIPT_NAME']}>"
			. "		<DIV CLASS='fsubmit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></DIV>"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='label'>Mindmeld Cache Directory</SPAN>"
			. "			<SPAN CLASS='formw'><INPUT TYPE=TEXT NAME=cache VALUE='$cache' SIZE=40/></SPAN>"
			. "			<SPAN CLASS='warn'>" . $msg['cache'] or '' . "</SPAN>"						
			. "			<SPAN CLASS='descr'><P>Verify that the <em>filesystem path</em> to Mindmeld's <b>cache</b> directory "
			. "				is correct and that your web server has permission to <em>write</em> to the directory and its contents."
			. "				By default, Mindmeld cache directory is located in the Mindmeld home directory in <em>mindmeld/cache</em>, "
			. "				but on a high-use system where performance is a concern, it may be wise to move the cache to a different "
			. "				physical disk drive."
			. "				You can change the cache location at any time by changing the <tt>cache</tt> entry in the <em>mindmeld.config</em> "
			. "				file in your Mindmeld home directory.</P>"
			. "			</SPAN>\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='label'>ADODB Home Directory</SPAN>"
			. "			<SPAN CLASS='formw'><INPUT TYPE=TEXT NAME=adodb_path VALUE='$adodb_path' SIZE=40/></SPAN>"
			. "			<SPAN CLASS='warn'>" . $msg['adodb_path'] or '' . "</SPAN>"
			. "			<SPAN CLASS='descr'><P>Verify that the <em>filesystem path</em> to the <b>ADODB library</b> "
			. "				is correct. The ADODB library directory should contain a file called <tt>adodb.inc.php</tt>."
			. "				You can change the location of the ADODB library at any time by changing the <tt>adodb</tt>"
			. "				entry in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
			. "			</SPAN>\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='submit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></SPAN>"
			. "		</DIV>\n"
			. "	</FORM>"
			. "</DIV>";

		return $html;
	}
	
	/**
	* Build HTML for Step 20
	*
	* Validate the following items: include, cache, sql_cache,
	* and adodb_path. Collect information for: interface_selected
	* and rdbms_type.
	*
	* @todo: rdbms_type form should examine install directory for available platforms and display as a list.
	*
	* @return string	HTML
	*/
	function step20() {
		
		$success = TRUE;
		
		// Validate data from the last step
		
		$regex = '/^(.*)\/*$/';
		
		$this->set_install( 'home', preg_replace( '/\/$/', '', $this->get_install( 'home' ) ) );
		$home = $this->get_install( 'home' );
		
		// Validate mindmeld home
		$check = array( $home => 'WRITE' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'home', $err );
			$success = FALSE;
		} else {
			$this->check( 'home' );
		}
		
		// Validate webhome
		$webhome = $this->get_install( 'webhome' );
		$check = array( $webhome => 'WRITE' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'webhome', $err );
			$success = FALSE;
		} else {
			$this->check( 'webhome' );
		}
		
		// Not currently checking site
		$this->check( 'site' );

		// Set the secondary paths
		if( ! $this->get_install( 'include' ) ) $this->set_install( 'include', "$home/include" );
		if( ! $this->get_install( 'cache' ) ) $this->set_install( 'cache', "$home/cache" );
		if( ! $this->get_install( 'sql_cache' ) ) $this->set_install( 'sql_cache', $this->get_install( 'cache' ) . "/sql_cache" );
		if( ! $this->get_install( 'adodb_path' ) ) $this->set_install( 'adodb_path', "$home/opensource/adodb" );

		
		// Validate include path
		$include = $this->get_install( 'include' );
		$check = array( $include => 'READ' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'include', $err );
			$success = FALSE;
		} else {
			$this->check( 'include' );
		}
		
		// Validate cache
		$cache = $this->get_install( 'cache' );
		$check = array( $cache => 'WRITE' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'cache', $err );
			$success = FALSE;
		} else {
			$this->check( 'cache' );
		}
		
		// Validate sql_cache
		$sql_cache = $this->get_install( 'sql_cache' );
		$check = array( $sql_cache => 'WRITE' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'sql_cache', $err );
			$success = FALSE;
		} else {
			$this->check( 'sql_cache' );
		}
		
		// Validate adodb
		$adodb_path = $this->get_install( 'adodb_path' );
		$check = array( $adodb_path => 'READ' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'adodb_path', $err );
			$success = FALSE;
		} else {
			$this->check( 'adodb_path' );
		}
		
		// Fail out if any of the validations fail
		if( !$success ) return NULL;
		
		
		// Initialize the form for the next step
		
		// Set the interface type
		if( $this->get_install( 'interface_selected' ) ) {
			$interface_selected = $this->get_install( 'interface_selected' );
		} else {
			$interface_selected = "HTML";
		}
		
		// Set the database type
		if( $this->get_install( 'rdbms_type' ) ) {
			$rdbms_type = $this->get_install( 'rdbms_type' );
		} else {
			$rdbms_type = "postgres7";
		}
		
		if( $this->messages ) {
			foreach( $this->messages as $key => $message ) {
				$msg[$key] = '<FONT COLOR=#AA0000>' . $message . '</FONT><BR/>';
			}
		}
		
		if( !isset( $msg ) ) $msg = array( 'interface_selected' => '', 'rdbms_type' => '' );
		
		// Collect data for next step
		// Use heredoc to build the HTML
	$istep = 'environment';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";
		$html .= "<SPAN CLASS='page_prompt'>" . $mmiStrings[$istep]['prompt'] . "</SPAN>";

		$html .= 
			  "<DIV CLASS='form'>\n"
			. "	<FORM METHOD=POST ACTION={$_SERVER['SCRIPT_NAME']}>"
			. "		<DIV CLASS='fsubmit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></DIV>"
		
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='label'>User Interface</SPAN>"
			. "			<SPAN CLASS='formw'>"
			. "				<SELECT CLASS='element_entity' NAME=interface_selected VALUE='$interface_selected'>"
			. "					<OPTION SELECTED VALUE='smarty' />HTML"
			. "					<!-- OPTION VALUE=postnuke/>Postnuke -->"
			. "				</SELECT>"
			. "			</SPAN>"
			. "			<SPAN CLASS='warn'>{$msg['interface_selected']}</SPAN>"
			. "			<SPAN CLASS='descr'><P>Select the user interface you will use with Mindmeld.<P>"
			. "				<P><B>HTML</B> will install a simple frames-based interface for searching, contributing, "
			. "				and administering content.<P>"
			. "				<!--P><B>Postnuke</B> will install the modules that integrate Mindmeld into a Postnuke web portal "
			. "				(See the <A HREF='http://postnuke.com'>Postnuke web site</A> for information about Postnuke.)</P -->"
			. "				<P>You can change the interface type at any time by changing the <tt>interface</tt>"
			. "				entry in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
			. "			</SPAN>\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='label'>Database Platform</SPAN>"
			. "			<SPAN CLASS='formw'>"
			. "				<SELECT NAME=rdbms_type VALUE='$rdbms_type'>"
			. "					<OPTION SELECTED VALUE='mysql' />MySQL"
			. "					<OPTION VALUE='postgres7' />PostgreSQL"
			. "					<OPTION VALUE='oci8' />Oracle"
			. "					<OPTION VALUE='mssql' />Microsoft SQL Server"
			. "				</SELECT>"
			. "			</SPAN>"
			. "			<SPAN CLASS='warn'>{$msg['rdbms_type']}</SPAN>"
			. "			<SPAN CLASS='descr'><P>Select the database platform that Mindmeld should install to. "
			. "				<em>If you are installing the <B>open source</B> version of Mindmeld, you <b>must</b> "
			. "				select <b>MySQL</b> as the database platform or Mindmeld will not work!</em>"
			. "				You can change the database platform type at any time by changing the <tt>dbType</tt>"
			. "				entry in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
			. "			</SPAN>\n"
			. "		</DIV>\n"
			
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='submit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></SPAN>"
			. "		</DIV>"
			. "	</FORM>"
			. "</DIV>";

		return $html;
	}
	
	/**
	* Build HTML for Step 30
	*
	* Validate the following items: interface_selected, rdbms_type.,
	* Collect information for: special iface/rdbms reqs, adodb_loaded, and
	* SQL parameters.
	*
	* @return string	HTML
	*/
	function step30() {
		
		$success = TRUE;
		
		// Validate data from the last step
		
		$regex = '/^(.*)\/?$/';
		$home = preg_replace( $regex, '\1/', trim( $this->get_install( 'home' ) ) );
		$cache = preg_replace( $regex, '\1/', trim( $this->get_install( 'cache' ) ) );
		$interface_selected = preg_replace( $regex, '\1', trim( $this->get_install( 'interface_selected' ) ) );
		
		// Validate interface_type by making sure path exists
		$interface_path = "{$home}interfaces/$interface_selected";
		$check = array(  $interface_path => 'READ' );
		if( $err = $this->validatePath( $check ) ) {
			$this->set_message( 'interface_selected', $err );
			$success = FALSE;
		} else {
			$this->check( 'interface_selected' );
		}
		
		// Fail out if any of the validations fail
		if( !$success ) return NULL;
		
		
		// Initialize the form for the next step
		$ifc = $this->get_install( 'interface_selected' );
		
		// Interface-specific items
		switch( $this->get_install( 'interface_selected' ) ) {
			
			case "smarty":
			
			// Path to Smarty
			if( $this->get_install( 'smarty' ) ) {
				$smarty_path = $this->get_install( 'smarty' );
			} else {
				$smarty_path = "{$home}opensource/smarty";
			}
			// Validate Path to Smarty
			$check = array(  $smarty_path => 'READ' );
			if( $err = $this->validatePath( $check ) ) {
				$this->set_message( 'smarty', $err );
				$success = FALSE;
			} else {
				$this->check( 'smarty' );
			}
			
			// Path to Smarty templates
			if( $this->get_install( 'templates' ) ) {
				$templates = $this->get_install( 'templates' );
			} else {
				$templates = "{$home}interfaces/smarty/templates";
			}
			// Validate Path to Smarty templates
			$check = array(  $templates => 'READ' );
			if( $err = $this->validatePath( $check ) ) {
				$this->set_message( 'templates', $err );
				$success = FALSE;
			} else {
				$this->check( 'templates' );
			}
			
			// Path to Smarty compiled templates
			if( $this->get_install( 'templates_c' ) ) {
				$templates_c = $this->get_install( 'templates_c' );
			} else {
				$templates_c = "{$cache}templates_c";
			}
			// Validate Path to Smarty templates
			$check = array(  $templates_c => 'WRITE' );
			if( $err = $this->validatePath( $check ) ) {
				$this->set_message( 'templates_c', $err );
				$success = FALSE;
			} else {
				$this->check( 'templates_c' );
			}
			
			break;
		}
		
		// Set RDBMS host
		if( $this->get_install( 'sqlHost' ) ) {
			$sqlHost = $this->get_install( 'sqlHost' );
		} else {
			$sqlHost = "localhost";
		}
		
		// Set RDBMS port
		if( $this->get_install( 'sqlPort' ) ) {
			$sqlPort = $this->get_install( 'sqlPort' );
		} else {
			$sqlPort = "";
		}
		
		// Set Database name
		if( $this->get_install( 'sqlName' ) ) {
			$sqlName = $this->get_install( 'sqlName' );
		} else {
			$sqlName = "mindmeld";
		}
		
		// Set Database user
		if( $this->get_install( 'sqlUser' ) ) {
			$sqlUser = $this->get_install( 'sqlUser' );
		} else {
			$sqlUser = "mindmeld";
		}
		
		// Set Database password
		if( $this->get_install( 'sqlPass' ) ) {
			$sqlPass = $this->get_install( 'sqlPass' );
		} else {
			$sqlPass = "";
		}
		
		if( $this->messages ) {
			foreach( $this->messages as $key => $message ) {
				$msg[$key] = '<FONT COLOR=#AA0000>' . $message . '</FONT><BR/>';
			}
		}
		
		if( !isset( $msg ) ) $msg = array( 'sqlHost' => '', 'sqlPort' => '', 'sqlName' => '', 
			'sqlUser' => '', 'sqlPass' => '' );
			
		// Collect data for next step
		// Use heredoc to build the HTML
		$adodb_include_path = preg_replace( $regex, '\1/', trim( $this->get_install( 'adodb_path' ) ) );
		
		$istep = 'connection';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";
		$html .= "<SPAN CLASS='page_prompt'>" . $mmiStrings[$istep]['prompt'] . "</SPAN>";

$html .= 
	  "<DIV CLASS='form'>\n"
	. "	<FORM METHOD=POST ACTION={$_SERVER['SCRIPT_NAME']}>"
	. "		<DIV CLASS='fsubmit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></DIV>"
		
	. "		<DIV CLASS='row'>\n"
	. "			<SPAN class='label'>Host and Port</SPAN>"
	. "			<SPAN CLASS='formw'>\n"
	. "				<INPUT TYPE=TEXT NAME=sqlHost VALUE='$sqlHost' SIZE=12/> : "
	. "				<INPUT TYPE=TEXT NAME=sqlPort VALUE='$sqlPort' SIZE=4/>"
	. "			</SPAN>\n"
	. "			<SPAN CLASS='warn'>{$msg['sqlHost']}</SPAN>"
	. "			<SPAN CLASS='warn'>{$msg['sqlPort']}</SPAN>"
	. "			<SPAN CLASS='descr'><P>The <STRONG>Host</STRONG> is the name of the server where the database will reside. Unless "
	. "				your database is running on a non-standard port, the <stron>port</stron> field may be left empty. "
	. "				You can change the host and port at any time by changing the <tt>dbHost</tt> and <tt>dbPort</tt> "
	. "				entries in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
	. "			</SPAN>\n"
	. "		</DIV>\n"
	
	. "		<DIV CLASS='row'>\n"
	. "			<SPAN class='label'>Database Name</SPAN>"
	. "			<SPAN CLASS='formw'><INPUT TYPE=TEXT NAME=sqlName VALUE='$sqlName' SIZE=20/></SPAN>"
	. "			<SPAN CLASS='warn'>{$msg['sqlName']}</SPAN>"
	. "			<SPAN CLASS='descr'><P>The <STRONG>Name</STRONG> is the name of new Mindmeld database instance. If possible, "
  	. "				the Mindmeld installer will create the database. For some RDBMS', you may need to request that your "
  	. "				database administrator (DBA) create the database for you."
	. "			 	You can change the database that Mindmeld accesses at any time by changing the <tt>dbName</tt> "
	. "				entry in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
	. "			</SPAN>\n"
	. "		</DIV>\n"
	
	. "		<DIV CLASS='row'>\n"
	. "			<SPAN class='label'>User</SPAN>"
	. "			<SPAN CLASS='formw'><INPUT TYPE=TEXT NAME=sqlUser VALUE='$sqlUser' SIZE=12/></SPAN>"
	. "			<SPAN CLASS='warn'>{$msg['sqlUser']}</SPAN>"
	. "			<SPAN CLASS='descr'><P>The <STRONG>User</STRONG> is the login of a database administrative user that has permission "
  	. "				to create new databases on your system. This user will be used by Mindmeld to connect to the database"
  	. "				during normal use."
  	. "				You can change the user at any time by changing the <tt>dbUser</tt> "
	. "				entry in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
	. "			</SPAN>\n"
	. "		</DIV>\n"
	
	. "		<DIV CLASS='row'>\n"
	. "			<SPAN class='label'>Password</SPAN>"
	. "			<SPAN CLASS='formw'><INPUT TYPE=PASSWORD NAME=sqlPass VALUE='$sqlPass' SIZE=12/></SPAN>"
	. "			<SPAN CLASS='warn'>{$msg['sqlPass']}</SPAN>"
	. "			<SPAN CLASS='descr'><P>The <STRONG>password</STRONG> is the password required for the user to log into the database."
	. "				You can change the password at any time by changing the <tt>dbPass</tt> "
	. "				entry in the <em>mindmeld.config</em> file in your Mindmeld home directory.</P>"
	. "			</SPAN>\n"
	. "		</DIV>\n"
	
	. "		<DIV CLASS='row'>\n"
	. "			<SPAN CLASS='submit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></SPAN>"
	. "		</DIV>\n"
	
	. "	<INPUT TYPE=HIDDEN NAME=adodb_include_path VALUE='$adodb_include_path'>"
	. "	</FORM>"
	. "</DIV>";
	
		return $html;
	}
	
	/**
	* Build and display installer HTML
	*
	* @return string	Installer HTML
	*/
	function showStatus() {
		
		$html = '';
		$html .= "<TABLE BORDER=2 CELLPADDING=2>\n";
		$html .= "<TR><TH>Item</TH><TH>Status</TH>\n";
		
		if( !$this->checklist ) return NULL;
		
		foreach( $this->checklist as $item => $checked ) {
			$html .= "<TR><TD>$item</TD>";
			if( $checked ) {
				$html .= "<TD>DONE</TD>";
			} elseif( $msg = $this->get_message( $item ) ) {
				$html .= "<TD>$msg</TD>";
			} else {
				$html .= "<TD>TBD</TD>";
			}
			$html .= "</TR>\n";
		}
		$html .= "</TABLE>\n";
		return $html;
	}
	
	function showInfo() {
		$html = "<H2 STYLE='margin-left: 1em'>Installation Status</H2>";
		$html .= "<TABLE BORDER=0 CELLPADDING=2 STYLE='margin-left: 2em; font-size: small'>\n";
		//$html .= "<TR><TH>Item</TH><TH>Status</TH></TR>\n";
		
		foreach( $this->requirements as $item => $req ) {
			$html .= "<TR>";
			
			if( isset( $this->checklist ) and array_key_exists( $item, $this->checklist ) and $this->checklist[$item] ) {
				$html .= "<TD><IMG SRC='../images/yes.gif' ALT='OK'></TD><TD>$item</TD>";
			} else {
				$html .= "<TD></TD><TD STYLE='color: gray'>$item</TD>";
			}
			$html .= "</TR>";
		}
		$html .= "</TABLE>";
		return $html;
	}
	
	function showInfoFull() {
		$html = '';
		$html .= "<TABLE BORDER=2 CELLPADDING=2>\n";
		$html .= "<TR><TH>Item</TH><TH>Req</TH><TH>Status</TH><TH>Value</TH><TH>Messages</TH></TR>\n";
		
		foreach( $this->requirements as $item => $req ) {
			$html .= "<TR><TD>$item</TD><TD>$req</TD>";
			
			if( isset( $this->checklist ) and array_key_exists( $item, $this->checklist ) and $this->checklist[$item] ) {
				$html .= "<TD BGCOLOR='#66FF66'>OK</TD>";
			} else {
				$html .= "<TD></TD>";
			}

			if( isset( $this->installValues ) and array_key_exists( $item, $this->installValues ) ) {
				$html .= "<TD>{$this->installValues[$item]}</TD>";
			} else {
				$html .= "<TD></TD>";
			}

			if( isset( $this->messages ) and array_key_exists( $item, $this->messages ) ) {
				$html .= "<TD>{$this->messages[$item]}</TD>";
			} else {
				$html .= "<TD></TD>";
			}
			$html .= "</TR>";
		}
		$html .= "</TABLE>";
		return $html;
	}
	
	/**
	* Initialze a requirements list
	*
	* Add checklist requirements. Default is the base requirements, but you can
	* also specify dependent requirements lists, such as SMARTY or POSTNUKE.
	*
	* @param string $listName	Name of checklist to add
	*/
	function initRequirements( $listName = 'BASE' ) {
		print "Listname is $listName<br>";
		$listName = strtoupper( $listName );
		
		switch( $listName ) {
			
			case "BASE":
				$requirements = array( 	'introduction' => 0, 
				'base' => 5,
				'home' => 20,
				'webhome' => 20,
				'site' => 20,
				'include' => 20,
				'cache' => 20,
				'sql_cache' => 20,
				'adodb_path' => 20,
				'interface_selected' => 30,
				'sqlHost' => 40,
				'sqlName' => 40,
				'sqlUser' => 40,
				'db_created' => 40,
				'db_populated' => 40,
				'webhome_config_written' => 60,
				'base_config_written' => 60 );
				break;
			
			case "SMARTY":
				$requirements = array(	'smarty' => 35,
				'templates' => 35,
				'templates_c' => 35 );
				break;
			
			case "POSTNUKE":
				$requirements = array( 'postnuke' => 37,
				'modules' => 37,
				'modules_written' => 38,
				'pn_configs_written' => 38 );
				break;
			
			default:
				$requirements = array( 	'introduction' => 0, 
				'base' => 5,
				'home' => 20,
				'webhome' => 20,
				'site' => 20,
				'include' => 20,
				'cache' => 20,
				'sql_cache' => 20,
				'adodb_path' => 20,
				'interface_selected' => 30,
				'sqlHost' => 40,
				'sqlName' => 40,
				'sqlUser' => 40,
				'db_created' => 40,
				'db_populated' => 40,
				'webhome_config_written' => 60,
				'base_config_written' => 60 );
				break;
		}
		
		// Append requirements to existing requirements
		$this->requirements = array_merge( ( $this->requirements ? $this->requirements : array() ), $requirements );
		
		// Add items to the checklist and set initial message
		foreach( $requirements as $requirement => $state ) {
			$this->check( $requirement, FALSE );
			$this->set_message( $requirement, '' );
		}
	}
	
	/**
	* check or uncheck an item on the checklist
	*
	* The checklist keeps track of what installation items are
	* complete or incomplete.
	*
	* @param string	$item		Item
	* @param boolean	$checked	Item state (TRUE|FALSE)
	*/
	function check( $item, $checked = TRUE ) {
		$this->checklist[$item] = $checked;
	}
	
	/**
	* Check status of a checklist item
	*
	* The checklist keeps track of what installation items are
	* complete or incomplete.
	*
	* @param string	$item		Item
	* @return boolean	Item state (TRUE|FALSE)
	*/
	function is_checked( $item ) {
		return $this->checklist[$item];
	}
	
	/**
	* Add a message to an item
	*
	* The messages list keeps track of what issues pertinent
	* to an item. Sending a NULL message clears the message
	* for the item.
	*
	* @param string	$item		Item
	* @param string	$msg		Message
	*/
	function set_message( $item, $msg = NULL ) {
		$this->messages[$item] = $msg;
	}
	
	/**
	* Return the message attached to an item
	*
	* The messages list keeps track of what issues pertinent
	* to an item. Sending a NULL message clears the message
	* for the item.
	*
	* @param string	$item		Item state (TRUE|FALSE)
	* @return string	$msg		Message or NULL on error
	*/
	function get_message( $item ) {
		if( isset( $this->messages ) and array_key_exists( $item, $this->messages ) ) {
			return $this->messages[$item];
		} else {
			return NULL;
		}
	}
	
	/**
	* Clear the messages list
	*
	* The messages list keeps track of what issues pertinent
	* to an item.
	*/
	function clear_messages() {
		if( isset( $this->messages ) ) $this->messages = array();
	}
	
	/**
	* Set the install value of an item
	*
	* The values list keeps the actual installation value
	* of an item. Sending a NULL value clears the value
	* for the item.print( $installer->GUIstep1() );
	*
	* @param string	$item		Item
	* @param string	$value		Value
	* @return mixed	Value
	*/
	function set_install( $item, $value = NULL ) {
		$this->installValues[$item] = $value;
		return $value;
	}
	
	/**
	* Return the install value attached to an item
	*
	* The values list keeps the actual installation value
	* of an item. Sending a NULL value clears the value
	* for the item.
	*
	* @param string	$item		Item
	* @return string	$value		Value
	*/
	function get_install( $item ) {
		if( isset( $this->installValues ) and array_key_exists( $item, $this->installValues ) ) {
			return $this->installValues[$item];
		} else {
			return NULL;
		}
	}
	
	/**
	* Set the config value of an item
	*
	* The config list keeps the actual configuration value
	* of an item. Sending a NULL value clears the value
	* for the item.
	*
	* @param string	$item		Item
	* @param mixed		$value		Value
	* @return mixed	Value
	*/
	function set_config( $item, $value = NULL ) {
		$this->configValues[$item] = $value;
		return $value;
	}
	
	/**
	* Return the install value attached to an item
	*
	* The config list keeps the actual configuration value
	* of an item. Sending a NULL value clears the value
	* for the item.
	*
	* @param string	$item		Item
	* @return string	$value		Value
	*/
	function get_config( $item ) {
		return $this->configValues[$item];
	}
	
	/**
	* Validate access to a file
	*
	* Validate the requested file or directory exists and has the
	* requested access permissions. Valid permissions are
	* READ: File is readable by script owner.
	* WRITE: File is writeable by script owner.
	* If no permission is passed, file is only checked for existance.
	*
	* @param string	$filename	File name or path
	* @param string	$perm		Requested permissions (R|W)
	* @return boolean	NULL if permitted access, else error message
	*/
	function inaccessible( $filename, $perm = 'ANY' ) {
		
		//$permission = strtoupper( $perm );
		$permission = preg_replace( '/(?i)^(A|(ANY).*)$/', 'ANY', $perm );
		$permission = preg_replace( '/(?i)^(R|(READ).*)$/', 'READ', $perm );
		$permission = preg_replace( '/(?i)^(W|(WRITE).*)$/', 'WRITE', $perm );
		
		$noExist = "$filename does not exist.";
		$noPerm = "No $permission access to $filename.";
		$badPerm = "$perm is not a valid permission. Use \"READ\" or \"WRITE\"";
		
		switch( $permission ) {
			
			case 'ANY':
				// Validate file existance
				if( !file_exists( $filename ) ) return $noExist;
				break;
			
			case 'READ':
				// Validate file existance
				if( !file_exists( $filename ) ) return $noExist;
				// Validate readable
				if( !is_readable( $filename ) ) return $noPerm;
				break;
			
			case 'WRITE':
				// Validate file existance
				if( !file_exists( $filename ) ) return $noExist;
				// Validate writeable
				if( !is_writeable( $filename ) ) return $noPerm;
				break;
			
			default:
				return $badPerm;
		}
		// File is accessible
		return NULL;
	}
	
	/**
	* Validate mindmeld supporting directories
	*
	* Validate mindmeld support directories exist and are
	* accessible. Error messages are written to the messages array.
	*
	* @param array		pathList		Path to check and perm to check for
	* @return boolean	NULL if successful, else error message
	*/
	function validatePath( $pathList ) {
		
		$errors = array();
		
		foreach( $pathList as $path => $perms ) {
			$path = preg_replace( '/\/*$/', '/', $path );
			if( $err = $this->inaccessible( $path, $perms ) ) {
				return $err;
			}
		}
		return NULL;
	}
	
	/**
	* Fetch defined form variables
	*
	* Fetch variable(s) from superglobals. Get it from the installer if it
	* doesn't exist, otherwise use it to set the installer value.
	* Also accept a validation format. This is designed to deal with simple
	* (not array or object) variable types.
	*
	* @param array	$varList	List of variable names and validation formats
	* @return array	List of variables and actual values or NULL if none exist
	*/
	function fetchVar( $varList ) {
		
		// Initialize the output array
		$returnList = array();
		
		// Cycle through the provided associative list. For each entry:
		// (1) run it through the validator, and if a value is returned
		// (2) use it to set the install value in the installer object, otherwise
		// return the current value from the installer object.
		foreach( $varList as $variable => $vformat ) {
			$value = $this->fetchSecureGlobal( $variable, $vformat );
			$value == NULL ? ( $value = $this->get_install( $variable ) ) : ( $this->set_install( $variable, $value ) );
			$returnList[$variable] =  $value;
		}
		if( sizeof( $returnList ) == 0 ) {
			return NULL;
		} else {
			return $returnList;
		}
	}
	
	/**
	* Fetch a global and make sure it's safe
	*
	* Fetch the requested global from the superglobals array and run
	* it through the security validator.
	*
	* @param string	$name	Global variable name
	* @param string	$policy	Security policy (see checkInput)
	* @return mixed	Value of variable or NULL if unsafe.
	*/
	function fetchSecureGlobal( $name, $policy ) {
		
		$safeValue = NULL;
		
		if( fetchGlobal( $name ) != NULL ) { 
			// global exists
			$quarentine = fetchGlobal( $name );
			$err = checkInput( $quarentine, $policy );
			if( $err ) { // Failed test!
				// Log warning
			} else {
				$safeValue = $quarentine;
			}
		}
		return $safeValue;
	}
}

// Define the installer wrapper class. If we know where the database abstraction
// layer is, we can load the database classes. Otherwise, we just wrap the base class.
//if( $MM_GLOBALS['adodb'] != NULL ) {
//if( defined( '_ADODB_LAYER' ) ) {

	/**
	 * Installer database access class
	 *
	 * TProvides database access methods and database-dependant
	 * methods common to all installer objects.
	 */
	class installer_db extends installer_base { // Installer with database functionality
		
		/*
		 * @var object	Database connection object.
		 */
		var $dbconn;
		
		/*
		 * @var object Contains access to all platform-specific SQL queries.
		 */
		var $sqlActions;
		
		/**
		 * Class constructor
		 *
		 * Initialize the database connection. Set initial parameters.
		 *
		 * @param boolean	$debug		SQL debug enabled (TRUE|FALSE)
		 * @param string		$dbType		Database platform
		 * @param string		$dbHost		Database host server
		 * @param string		$dbPort		Database host port
		 * @param string		$dbName		Database name
		 * @param string		$dbUser		Database user
		 * @param string		$dbPasswd	Database user password
		 * @return	object	Database connection object
		 */
		function init_db( 	$sqlDebug,	$dbType, $dbHost, $dbPort, $dbName, $dbUser, $dbPasswd ) {
			
			// Fail if we haven't loaded the DB Abstraction System
			global $foundAdodb;
			if( !$foundAdodb ) return FALSE;
			
			// Initialize connection information
			if ( $dbPort ) $dbHost .= ":$dbPort";
			
			//print( "<P>Connecting to $dbType $dbName on $dbHost as $dbUser with $dbPasswd</P>" );
			
			// Create the connection 
			$dbconn = &ADONewConnection( $dbType );
			$dbconn->debug = $sqlDebug;
			$dbconn->Connect( $dbHost, $dbUser, $dbPasswd );
			
			// Make the connection accessable as a class property
			$this->dbconn = $dbconn;
			
			return $dbconn;
		}
				
		/**
		* Build HTML for Step 40
		*
		* Validate the following items: SQL parameters. Attempt to create and populate
		* the database.
		*
		* @return string	HTML
		*/
		function step40() {
			
			// Fail if we haven't loaded the DB Abstraction System
			global $foundAdodb;
			if( !$foundAdodb ) return FALSE;
			
			$success = TRUE;
			
			// Validate data from the last step
			
			$regex = '/^(.*)\/?$/';
			$home = preg_replace( $regex, '\1/', trim( $this->get_install( 'home' ) ) );
			$dbType = $this->get_install( 'rdbms_type' );
			$dbHost = $this->get_install( 'sqlHost' );
			$dbPort = $this->get_install( 'sqlPort' );
			$dbName = $this->get_install( 'sqlName' );
			$dbUser = $this->get_install( 'sqlUser' );
			$dbPass = $this->get_install( 'sqlPass' );
			
			$this->init_db( FALSE, $dbType, $dbHost, $dbPort, $dbName, $dbUser, $dbPass );
			
			// Create the database
			$dbconn = &$this->dbconn;
			
			$dict = NewDataDictionary( $dbconn );
			$dbconn->Connect( $dbHost, $dbUser, $dbPass );
			$sqlCreate = $dict->CreateDatabase( $dbName );
			$dbconn->Execute( $sqlCreate[0] );
			$dict->SetSchema( $dbName );
			
			// Populate the database
			$dbconn->Connect( $dbHost, $dbUser, $dbPass, $dbName );
			$schema = new adoSchema( $dbconn );
			
			print "<SPAN CLASS='dump'>";
			$sql = $schema->ParseSchema( "build_mindmeld.xml" );
			$success = $schema->ExecuteSchema( $sql );
			$schema->Destroy();
			print "</SPAN>";
			
			switch( $success ) {
			
				case 0:	// Create/Build failed
					$this->set_message( 'db_populated', "Database creation and build failed." );
					break;
				case 1:	// Create/Build completed with errors
					$this->set_message( 'db_populated', "Errors occurred while building the database" );
					break;
				case 2:	// Create/Build successful
					$this->check( 'db_created' );
					$this->check( 'db_populated' );
					$this->check( 'sqlUser' );
					$this->check( 'sqlHost' );
					$this->check( 'sqlName' );
					break;
			}
			
			// Collect data for next step
			// Use heredoc to build the HTML
			$adodb_include_path = preg_replace( $regex, '\1/', trim( $this->get_install( 'adodb_path' ) ) );
		
		$istep = 'dbcreated';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";
		$html .= "<SPAN CLASS='page_prompt'>" . $mmiStrings[$istep]['prompt'] . "</SPAN>";

		$html .= 
		  	  "<DIV CLASS='form'>\n"
  			. "	<FORM METHOD=POST ACTION={$_SERVER['SCRIPT_NAME']}>\n"
  			. "		<DIV CLASS='fsubmit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></DIV>"
		
			. "		<DIV CLASS='row'>\n"
			. "			<SPAN CLASS='submit'><INPUT TYPE=SUBMIT NAME=submit VALUE=Continue /></SPAN>\n"
			. "		</DIV>\n"
			. "	<INPUT TYPE=HIDDEN NAME=adodb_include_path VALUE='$adodb_include_path'>"
			. "	</FORM>\n"
			. "</DIV>\n";
			
			return $html;
		}
		
		/**
		* Build HTML for Step 60
		*
		* Write the config files and back up the installer files. Copy header.inc to webhome.
		*
		* @return string	HTML
		* @todo Copy install directory to cache and remove from webhome
		*/
		function step60() {
			
			// Fail if we haven't loaded the DB Abstraction System
			global $foundAdodb;
			if( !$foundAdodb ) return FALSE;
			
			$success = TRUE;
			
			$interface_selected = preg_replace( '/\/$/', '', $this->get_install( 'interface_selected' ) );
			$this->set_install( 'interface_selected', $interface_selected );
			
			// Write the webhome config file
			$item = 'webhome_config_written';
			$configValues = array( 	'home' => realpath( $this->get_install( 'home' ) ),
			'interface' => $this->get_install( 'interface_selected' ),
			'theme' => 'arsgreen',
			'debug' => -1,
			'sqlDebug' => FALSE );
			if( $err = $this->writeConfig( $item, $this->get_install( 'webhome' ), $configValues ) ) {
				$this->set_message( $item, $err );
			} else {
				$this->check( $item );
			}
			
			// Write the main config file
			$item = 'base_config_written';
			$configValues = array( 	'home' => $this->get_install( 'home' ),
			'webhome' => $this->get_install( 'webhome' ),
			'site' => $this->get_install( 'site' ),
			'cache' => $this->get_install( 'cache' ),
			'dbType' => $this->get_install( 'rdbms_type' ),
			'dbName' => $this->get_install( 'sqlName' ),
			'dbHost' => $this->get_install( 'sqlHost' ),
			'dbPort' => $this->get_install( 'sqlPort' ),
			'dbUser' => $this->get_install( 'sqlUser' ),
			'dbPass' => $this->get_install( 'sqlPass' ),
			'adodb' => $this->get_install( 'adodb_path' ),
			'sql_cache_dir' => $this->get_install( 'sql_cache' )
			);
			if( $err = $this->writeConfig( $item, $this->get_install( 'home' ), $configValues ) ) {
				$this->set_message( $item, $err );
			} else {
				$this->check( $item );
			}
			
			// Copy header.inc to webhome.
			$srcfile = $this->get_install( 'home' ) . '/include/header.inc';
			$destfile = $this->get_install( 'webhome' ) . '/header.inc';
			copy( $srcfile, $destfile );
			
			// Wrap up
			// Use heredoc to build the HTML
			$home = $this->get_install( 'home' );
			$webhome = $this->get_install( 'webhome' );
			$site = $this->get_install( 'site' );
			
		$istep = 'summary';
		global $mmiStrings;
		$html = $mmiStrings['app']['title'];
		$html .= "<P CLASS='page_title'>" . $mmiStrings[$istep]['title'] ."</P>";

$html .= 
			  "<P>Mindmeld has written the following configuration files:</P>"
			. "<P><B>Default Configuration:</b> <TT>$home/mindmeld.config</TT></P>"
			. "<P>The default configuration file contains basic configuration information for "
			. "the Mindmeld installation on this host. Backup and edit this file if you need to "
			. "change database connection information, system paths, or other system-wide options.</P>"
			
			. "<P><B>Local Configuration:</b> <TT>$webhome/mindmeld.config</TT></P>"
			. "<P>The local configuration file contains configuration information that is specific to " 
			. "the given Mindmeld instance. Backup and edit this file if you need to change anything. "
			. "<EM>Storing database connection information in the Local Configuration file constitues "
			. "a serious security breach!</EM></P>" 
			. "<P><STRONG>Note:</STRONG> Options specified in the <EM>Local Configuration</EM> file always "
			. "override options specified in the <EM>Default Configuration.</EM></P>";

$html .= "<SPAN CLASS='security_title'>Mindmeld Security Notice</SPAN>";
$html .= "<SPAN CLASS='security'>" . $mmiStrings[$istep]['security'] . "</SPAN>";

$html .= 
			  "<H2> Getting Started... </H2>"
			. "<P><A HREF='$site'>Browse to your new Mindmeld System.</A></P>"
			. "<P>Thank your for using Mindmeld!</P>";

$html .=
			  "<DIV CLASS='form'>\n"
  			. "	<FORM METHOD=POST ACTION='$site'>\n"
  			. "		<DIV CLASS='fsubmit'><INPUT TYPE=SUBMIT NAME=submit VALUE='Go Mindmeld!' /></DIV>"
			. "	</FORM>\n"
			. "</DIV>\n";
			
			return $html;
		}
	}
//}



/**
* Bypass gettext for systems to do not support it
*
* Check for gettext and provide a dummy gettext function if it isn't available.
*
* @param string		$passthru			Untranslated text
* @return string	Passthrough text
*/
if ( ! function_exists('_') ) {
	function _( $passthru = NULL ) {
		return $passthru;
	}
}

/**
* Provide security for user input
*
* Provide security for user input from all sources. Format for
* the input value is: "passed value, test sequence" where the
* sequence is a pipe-delimited list of keywords and optional
* arguments.
*
* The checklist sequences are:
*	type=INT|FLOAT|STRING
*	min={INTEGER|FLOAT}
*	max={INTEGER|FLOAT}
*	minlength={INTEGER}
*	maxlength={INTEGER}
*
* Function operates on the original string and returns
* NULL if the string is acceptable or an error message if
* if wasn't.
*
* @param mixed		$value			Data to check (by reference)
* @param string		$test			Tests to be performed
* @return string	Error message or NULL
*/
function checkInput( &$value, $test_sequence ) {
	// Store the original value so we know how it changed.
	$rawValue = $value;
	$value = NULL;
	
	$errMsg = array(	'INT_FAIL' => "Not an integer value",
	'FLOAT_FAIL' => "Not a float value",
	'BOOLEAN_FAIL' => "Not a boolean value",
	'MIN_CRITERIA' => "Bad criterion for minimum value",
	'MIN_FAIL' => "Value less than minimum",
	'MAX_CRITERIA' => "Bad criterion for maximum value",
	'MAX_FAIL' => "Value greater than maximum",
	'MINLEN_CRITERIA' => "Bad criterion from minimum length",
	'MINLEN_FAIL' => "Value less than minimum length",
	'MAXLEN_CRITERIA' => "Bad criterion from maximum length",
	'MAXLEN_FAIL' => "Value exceeds than maximum length",
	'REGEX_FAIL' => "Value fails regex match" );
	
	// Parse the test sequence into individual tests
	$testList = explode( "|", $test_sequence );
	
	// Apply each test
	foreach( $testList as $test ) {
		
		// Break test into key and sequence
		list( $key , $criteria ) = explode( "=", $test, 2 );
		$key = strtoupper( $key );
		
		// Handle any test aliases
		if ( $key == "MINLEN" ) $key = "MINLENGTH";
		if ( $key == "MAXLEN" ) $key = "MAXLENGTH";
		
		// Select the test
		switch( $key ) {
			case "TYPE":	// Data type
			$criteria = strtoupper( $criteria );
			
			if( $criteria == "INT" ) {	// INT (integer) types
			// Regex match begin line followed by optional - or +
			// followed by one or more digits followed by end line.
			$goodMatch = preg_match( "/^[-+]?[0-9]+$/", $rawValue );
			if( !$goodMatch ) {
				$value = NULL;
				return $errMsg['INT_FAIL']; // Fail out
			}
			}
			
			if( $criteria == "FLOAT" ) {	// FLOAT (float) types
			// Regex match begin line followed by optional - or +
			// followed by one or more digits followed by optional
			// decimal point followed by one or more digits followed by
			// optional E followed by one or more digits followed by end line.
			$goodMatch = preg_match( "/^[-+]?[0-9]+(\.[0-9]+)?([-+]?[eE][0-9]+)?$/", $rawValue );
			if( !$goodMatch ) {
				$value = NULL;
				return $errMsg['FLOAT_FAIL'];	// Fail out
			}
			}
			
			if( $criteria == "BOOLEAN" ) {	// Boolean types
			// Regex match case-insensitive begin line followed
			// by TRUE or FALSE followed by end line.
			$goodMatch = preg_match( "/(?i)^true|false$/", $rawValue );
			if( !$goodMatch ) {
				$value = NULL;
				return $errMsg['BOOLEAN_FAIL'];	// Fail out
			}
			}
			break;
			
			case "MIN":	// Numeric minimum value
			// Criteria is the minimum allowed value
			
			// Regex match begin line followed by optional - or +
			// followed by one or more digits followed by optional
			// decimal point followed by one or more digits followed by
			// optional E followed by one or more digits followed by end line.
			if( !preg_match( "/^[-+]?[0-9]+(\.[0-9]+)?([eE][0-9]+)?$/", $criteria ) ) { // bad criterion
			return $errMsg['MIN_CRITERION'];
			} else {
				if( $rawValue < $criteria ) {
					$value = NULL;
					return $errMsg['MIN_FAIL']; // Failed test
				}
			}
			break;
			
			case "MAX":	// Numeric maximum value
			// Criteria is the maximum allowed value
			
			// Regex match begin line followed by optional - or +
			// followed by one or more digits followed by optional
			// decimal point followed by one or more digits followed by
			// optional E followed by one or more digits followed by end line.
			if( !preg_match( "/^[-+]?[0-9]+(\.[0-9]+)?([eE][0-9]+)?$/", $criteria ) ) { // Bad criterion
			return $errMsg['MAX_CRITERIA'];
			} else {
				if( $rawValue > $criteria ) {
					$value = NULL;
					return $errMsg['MAX_FAIL'];	// Failed test
				}
			}
			break;
			
			case "MINLENGTH":	// String minimum length
			// Criteria is the minimum number of allowed characters
			if( !preg_match( "/^[0-9]+$/", $criteria ) ) { // bad criterion
			return $errMsg['MINLEN_CRITERIA'];
			} else { // Good criterion
			if( strlen( $rawValue ) < $criteria ) {
				$value = NULL;
				return $errMsg['MINLEN_FAIL']; // Too short
			}
			}
			break;
			
			case "MAXLENGTH":	// String maximum length
			// Criteria is the minimum number of allowed characters
			if( !preg_match( "/^[0-9]+$/", $criteria ) ) { // Bad criterion
			return $errMsg['MAXLEN_CRITERIA'];
			} else { // Good criterion
			if( strlen( $rawValue ) > $criteria ) {
				$value = NULL;
				return $errMsg['MAXLEN_FAIL'];	// Too long
			}
			}
			break;
			
			default:
		}
	}
	
	$value = $rawValue;
	return NULL;
}

/**
* remove_magic_quotes from variable
*
* Checks the setting of magic_quotes_gpc and acts accordingly, stripping
* quotes and escapes from globals arrays as necessary.
*
* Thanks to gordon at kanazawa dot ac dot jp for this chunk of code.
*/
function remove_magic_quotes( &$x ) {
	if( is_array( $x ) ) {
		while( list( $key, $value ) = each( $x ) ) {
			if( $value ) remove_magic_quotes( $x[$key] );
		}
	} elseif( ini_get( 'magic_quotes_sybase' ) ) {
		$x = preg_replace( "/''/", "'", $x );
	} elseif( get_magic_quotes_runtime() ) {
		$x = preg_replace( "/\\\"/", '"', $x );
	} elseif( get_magic_quotes_gpc() ) {
		$x = stripslashes( $x );
	}
}

/**
* Fetch a global variable from the Superglobals array
*
* @param mixed		$varName	Name of variable to fetch
* @param mixed		$default	Value to return for nonexistant variable
* @return mixed		Value of global variable
*/
function fetchGlobal( $varName, $default = NULL ) {
	
	if ( array_key_exists( $varName, $_SERVER ) ) {
		$varValue = $_SERVER[$varName];
	} elseif ( array_key_exists( $varName, $_COOKIE ) ) {
		$varValue = $_COOKIE[$varName];
	} elseif ( array_key_exists( $varName, $_POST ) ) {
		$varValue = $_POST[$varName];
	} elseif ( array_key_exists( $varName, $_GET ) ) {
		$varValue = $_GET[$varName];
	} elseif ( array_key_exists( $varName, $_ENV ) ) {
		$varValue = $_ENV[$varName];
	} else {
		$varValue = $default;
	}
	
	return $varValue;
}
?>
