<?PHP
// Copyright (c) 2003 ars Cognita, Inc., all rights reserved
/********************************************************************************

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * Mindmeld graphical installer
 *
 * @author		$Author: richtl $
 * @version		$Revision: 1378 $
 * @copyright	2003, ars Cognita, Inc., all rights reserved
 *
 * @package installer
 */

set_time_limit( 8000 );

define( 'SESS_NAME', 'install' );

// Get the database layer if it's set
if( isset( $_POST['adodb_include_path'] ) ) {
	/**
 	 * Database abstraction library
 	 */
	$foundAdodb = include( $_POST['adodb_include_path'] . "adodb.inc.php" );
	if( $foundAdodb ) include( $_POST['adodb_include_path'] . "adodb-xmlschema.inc.php" );
}

/**
 * Installer utilities library
 */
require_once( 'install.inc' );

// Create the session
session_name( SESS_NAME );
session_start();

// Translate submit buttons from forms
if( !isset( $_POST['submit'] ) ) {
	// Destroy old session
	session_destroy();
	unset( $_SESSION['installer'] );
}

// Process incoming data
remove_magic_quotes($_POST);
set_magic_quotes_runtime(0);

if( isset( $_SESSION['installer'] ) ) {
	// Object exists in session
	$installer = &$_SESSION['installer'];
} else {
	// No object in session. Create a new one.
	$installer = new installer_db();
	$installer->initRequirements();
	$_SESSION['installer'] = &$installer;
}
	
if( FALSE ) {
	// Set up the correct object. Use the non-database oriented installer until
	// database information is available. Then copy the non-db object to the db
	// object and use only the db object from there on.
	if( isset( $_SESSION['installer'] ) ) {
		// Object exists in session
		if( !class_exists( 'installer_db' ) ) { 
			// Not using database installer yet
			$installer = &$_SESSION['installer'];
		} else {
			// Database installer is loaded
			if( is_a( $_SESSION['installer'], 'installer_db' ) ) {
				// Object is a DB installer object
				$installer = &$_SESSION['installer'];
			} else {
				// Convert the non-DB object to a DB object.
				$temp = &$_SESSION['installer'];
				$installer = new installer_db();
				$installer->requirements = $temp->requirements;
				$installer->checklist = $temp->checklist;
				$installer->messages = $temp->messages;
				$installer->installValues = $temp->installValues;
				$installer->configValues = $temp->configValues;
				$installer->lastState = $temp->lastState;
				$_SESSION['installer'] = &$installer;
			}
		}
	} else {
		// No object in session. Create a new one.
		$installer = new installer_base();
		$installer->initRequirements();
		$_SESSION['installer'] = &$installer;
	}
}

// Send HTML head after all session headers have been sent
print $mmiStrings['app']['head'];

// Define form variables that should be fetched from the superglobals array.
// Then extract the returned array into local variables of the same name.
if( isset( $_POST['submit'] ) ) {
	$varList = array( 	'home' => VALID_PATH, 'home_alt' => VALID_PATH, 'site' => VALID_PATH, 'webhome' => VALID_PATH,
						'include' => VALID_PATH, 'cache' => VALID_PATH, 'sql_cache' => VALID_PATH,
						'adodb_path' => VALID_PATH, 'interface_selected' => VALID_STRING,
						'rdbms_type' => VALID_STRING, 'sqlHost' => VALID_STRING, 'sqlPort' => VALID_PORTNUM,
						'sqlName' => VALID_STRING,
						'sqlUser' => VALID_STRING, 'sqlPass' => VALID_STRING, 'sqlAdminUser' => VALID_STRING,
						'sqlAdminPass' => VALID_STRING );
	
	extract( $installer->fetchVar( $varList ) );
	if( $home_alt != '' ) {
		$home = $home_alt;
		$installer->set_install( 'home', $home_alt );
	}
}

if( isset( $_POST['submit'] ) and $_POST['submit'] == 'Begin' ) $installer->check( 'introduction' );

// State Processing
$state = $installer->getState();
//showDebug( $state, "State is" );
//showDebug( $installer, "Installer" );
$installerType = get_class( $installer );

switch( $state ) {
	
	case 60:
	
		$html = $installer->step60();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	case 50:
	
		$html = $installer->step50();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	case 40:
		
		// Validate special information and SQL parameters
		// Create and populate the database
		$html = $installer->step40();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	case 30:
		
		// Validate interface and RDBMS platform
		// Request special information and SQL parameters
		$html = $installer->step30();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	case 20:
		
		// Validate supporting locations
		// Request interface and RDBMS platform
		$html = $installer->step20();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	case 10:
		
		// Validate base locations
		// Request supporting locations
		$html = $installer->step10();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	case 5:
		// Request base locations
		$html = $installer->step5();
		if( isset( $html ) ) {
			print $html;
			break;
		}
	
	default:
		
		// Introduction
		print $installer->step0();
}
$installer->clear_messages();

function showDebug( $var, $comment = '' ) {
	if( $comment ) print "<P><B>$comment</B></P>";
	print "<PRE>";
	print_r( $var );
	print "</PRE>";
}

/** Debug info. Turned off for now
print( "<HR>" );
print( $installer->showInfo() );
print( "<H2>Messages</H2>" );
print_r( $installer->messages );
**/

// HTML Page close
print $mmiStrings['app']['foot'];
?>
