<?PHP
// Copyright (c) 2001-2006 ars Cognita, Inc., all rights reserved
/********************************************************************************
		
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * Mindmeld API example
 * 
 * Example of how to search for information in Mindmeld using the Mindmeld 
 * API.
 * 
 * Created Mar 4, 2006 by richtl
 *  
 * @author		richtl
 * Last Editor:	$Author: richtl $
 * @version		$Revision: 1.85 $
 * 
 * @package examples
 */

/**
 * Configure Mindmeld
 */
$MM_GLOBALS['home'] = '/home/richtl/workspace/Beta-1_2_2/';
$MM_GLOBALS['cache'] = '/tmp/';
$MM_GLOBALS['adodb'] = '/home/richtl/workspace/Beta-1_2_2/opensource/adodb/';
$MM_GLOBALS['dbType'] = 'mysql';
$MM_GLOBALS['dbUser'] = 'root';
$MM_GLOBALS['dbPass'] = '';
$MM_GLOBALS['dbHost'] = 'localhost';
$MM_GLOBALS['dbName'] = 'mm122';

/**
 * Load Mindmeld libraries
 */
require_once( $MM_GLOBALS['home'] . 'include/utilities.inc' );
require_once( $MM_GLOBALS['home'] . 'include/mind.inc.php' );

// The mindmeld object creates the ADOdb database connection
// and provides basic methods for working with knowledge objects.
$mindmeld = new mindmeld( 
		NULL,
		$MM_GLOBALS['dbType'],
		$MM_GLOBALS['dbHost'],
		NULL,
		$MM_GLOBALS['dbName'],
		$MM_GLOBALS['dbUser'],
		$MM_GLOBALS['dbPass']
		);

/* The search process has four steps:
 * Step 1. Submit the user's search text to Mindmeld.
 * Step 2. Display a list of search results for the user to select from.
 *   They should also be able to modify and resubmit the search text.
 * Step 3. Display the selected result and obtain user feedback.
 * Step 4. Apply the feedback and return to Step 2.
 */
 
// Step 1. The search text is user's query, usually provided
// in an HTML form.
$searchtext = 'When should I look for moose?';
print "<h2>Submit the Search</h2>\n";
print "<p>The user submits <em>$searchtext</em></p><hr/>\n";

// Create a new mind (facade) object and use it to
// submit the query.
$mind = new mind();
if( !$queryId = $mind->ask( $searchtext ) )
	die( 'Failed to submit the search' );

// Step 2. Use the returned queryId to fetch the search results.
// Results are returned in descending order of relevance.
$results = $mind->answers( $queryId );

print "<h2>Relevant Answers</h2>\n";
print "<table>\n";
print "<tr><th>Thought ID</th><th>Thought Summary</th><th>Relevance</th></tr>\n";
foreach( $results as $answer ) {
	print "<tr><td>{$answer[0]}</td><td>{$answer[1]}</td><td>{$answer[2]}</td</tr>\n";
}
print "</table>\n";
print "<p><em>Let the user select one from the list to view.</em></p><hr/>\n";

// Step 3. Presume the user selects thought #2 from the above list. We need to fetch it
// and display it for them. We also need to know if the selected answer was relevant 
// to the question.) User feedback can be determined with YES/NO buttons, from a 
// "Print/Email this Answer" button, or any other reasonably reliable method.
$thoughtId = 2;
$thought = $mind->fetchThought( $thoughtId );
print "<h2>Selected Answer</h2>\n";
print "<p>{$thought->getSummary()}</p>\n";
print "<p>{$thought->getDetail()}</p>\n";
print "<p><em>Was it helpful? Provide buttons to get user feedback</em></p><hr/>\n";

// Step 4. Finally, provide Mindmeld with feedback from the user as to whether the answer
// was relevant. FEEDBACK IS REALLY IMPORTANT--IT'S WHAT ALLOWS MINDMELD TO LEARN.
// If the answer was relevant, set $feedback = MM_STRENGTHEN. If the answer wasn't 
// relevant, set $feedback = MM_WEAKEN. If the user isn't sure or doesn't answer,
// you can set $feedback = MM_IGNORE.
$feedback = MM_STRENGTHEN;
$mind->reinforceMemory( $thoughtId, $queryId, $feedback );

// Destroy the mindmeld object when we're finished with it.
$mindmeld->close();
?>
