<?PHP
// Copyright (c) 2001-2003 ars Cognita, Inc., all rights reserved
/********************************************************************************
	
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * HTML GUI to search for answers
 *  
 * @author		$Author: richtl $
 * @version		$Revision: 990 $
 *
 * @package interfaces
 * @subpackage smarty
 */
 
/**
 * Smarty general interface module
 */
require_once( $MM_GLOBALS['home'] . "interfaces/smarty/include/interface_smarty.inc"  );

/**
 * High-level Mindmeld module
 */
require_once( $MM_GLOBALS['home'] . "include/mind.inc.php" );

/**
* class searchPresentation
*
* The searchPresentation class provides methods for building HTML interfaces
* for querying the mindmeld system. These would usually be provided to "users."
*/
class searchPresentation {
	
	/**
	* @var string Submitted query string
	*/
	var $theQuestion;
	
	/**
	* @var integer Unique ID for this query
	*/
	var $queryId;
	
	/**
	* @var integer Unique ID for the selected thought
	*/
	var $thoughtId;
	
	/**
	* @var boolean TRUE if the user has provided feedback for the query
	*/
	var $haveFeedback;
	
	/**
	* @var boolean TRUE if this is an edit request
	*/
	var $edit;	
	
	/**
	* Class constructor
	*
	* Doesn't do anything at the moment.
	*/
	function searchPresentation() {
	}
	
	/**
	* Build and display the the HTML header
	*
	* Wrapper that calls the general mmsmarty_showHeader function from
	* interface_smarty.inc. This method takes a thought, checks its displayRaw
	* property to see if headers should be displayed, and if so, sends script
	* and style tags.
	*
	* @param object $thought thought object
	*/
	function showHeader( $thought ) {
		
		if( !$thought->displayRaw ) return NULL;
		
		$hdrObj = parseHtml( $thought->getDetail() );
		unset( $hdrObj->headerMetaTags );
		unset( $hdrObj->headerMiscTags );
				
		mmsmarty_showHeader( $hdrObj );
	}
		
	/**
	* Build and display the search query
	*
	* Part of the user interface for searching for content. This function gets the 
	* actual query text from the user.
	*/
	function showQuery () {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->assign( "theQuestion", $this->theQuestion );
		$template->display( "ask_showQuery.tpl" );
	}
	
	/**
	* Build and display the search results list
	*
	* Part of the user interface for searching for content. This function lists the 
	* possible matches for the user to choose from.
	*/
	function showResults( $results = NULL ) {
		
		global $urlSelf;
		
		// Build the links for the list
		strrpos( $urlSelf, '?' ) ? ( $link = $urlSelf."&" ) : ( $link = $urlSelf."?" );
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;
		$template->assign_by_ref( "baseLink", $link );
		$template->assign_by_ref( "answers", $results );
		$template->display( "ask_showResults.tpl" );
	}
	
	/**
	* Build and display the search feedback request
	*
	* Part of the user interface for searching for content. This function requests 
	* feedback from the user.
	*/
	function feedback( $haveFb = FALSE ) {
		
		// Create and display the HTML template.
		
		if ( !$haveFb ) {
			$template = new mmHtmlTemplate;
			$template->display( "ask_getFeedback_brief.tpl" );
		} else {
			$template = new mmHtmlTemplate;
			$template->display( "ask_haveFeedback.tpl" );
		}
	}
	
	/**
	* Build and display the selected answer
	*
	* Part of the user interface for searching for content. This function displays 
	* the selected thought to the user.
	*/
	function displayThought( $thought = NULL ) {
		
		if ( isset( $this->thoughtId ) ) {
			
			// Create and display the HTML template.
			$template = new mmHtmlTemplate;
			$htmlObj = parseHtml( $thought->getDetail() );
			
			if( $thought->displayRaw ) {	// Display headers and unfiltered HTML
				$detail = $htmlObj->body;
			} else {	
				// Display normal (filtered) HTML and no headers

				/*
				* PLUGIN HOOK: Filter to manipulate an answer prior to display as the result of a search
				*
				* Provides access to a thought object before the object is displayed.
				* Note: Changes to the thought are not permanent unless the plugin
				* explicitly calls the thought->updateThough() method.
				*
				* @param object thought object
				* @return object Filtered thought object
				*/
				$thought = $thought->pluginMgr->executePlugins( 'displayAnswerPrefilter', $thought );
				
				$detail = $thought->getDetail();
			}

			$template->assign_by_ref( "thoughtId", $this->thoughtId );
			$template->assign( "thoughtSummary", $thought->getSummary() );
			$template->assign_by_ref( "thoughtDetail", $detail );	
			$template->assign_by_ref( "haveFeedback", $this->haveFeedback );
			$template->assign_by_ref( "edit", $this->edit );
			$template->display( "ask_showAnswer.tpl" );
		}
	}
}

/**
* class mmSearch
*
* The mmSearch class provides methods for all the logic necessary for
* querying the mindmeld system.
*/
class mmSearch extends searchPresentation {
	
	/**
	* Execute the search
	*
	* Search for a list of possible matches. Return the results list as an array
	*
	* @return results array 	List of possible matches
	*/
	function performQuery() {
		
		// Create a new mind object.	
		$mind = new mind();
		
		// Run the query and dump the results into an array
		$this->queryId = $mind->ask( stripslashes( $this->theQuestion ) );
		$results = $mind->answers( $this->queryId );	
		
		return $results;
	}
	
	/**
	* Return a list of potential answers given a query ID
	*
	* Return a list of potential answers for a query that has already been processed.
	*
	* @param integer	$queryId		Query ID
	* @return array	Results array
	*/
	function getResults( $queryId ) {
		
		$mind = new mind();
		$results = $mind->answers( $queryId );
		
		return $results; 
	}
	
	/**
	* Get the selected answer
	*
	* Fetch the current thought by ID.
	*
	* @return	thought		Thought object
	*
	* TODO: This should return a pointer to the object
	*/
	function fetchThought( $thoughtId = NULL ) {
		
		if ( !isset( $this->thoughtId ) ) {
			$thought = NULL;
		} else {
			// Create a new thought object and fetch the thought summary and detail
			$thought = new thought();
			$thought->readThoughtFull( $this->thoughtId );
		}
		return $thought;
	}
	
	/**
	* Update the status of the query
	*
	* Update the status of the current query from New to Open, since the user
	* has shown interest in a possible answer.
	*
	* @param	string	$newStatus		New status of query
	* @return	query	Query object
	*
	* TODO: This should return a pointer to the object
	*/
	function updateQueryStatus( $newStatus = 'OPEN' ) {
		
		$query = NULL;
		
		if ( $this->queryId != NULL ) {
			
			$query = new query();
			$query->setQueryId( $this->queryId );
			$query->setStatus( $newStatus );
			$query->updateQuery();
		}
		return $query;
	}
}
?>
