<?PHP
// Copyright (c) 2003 ars Cognita, Inc., all rights reserved
/********************************************************************************

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * GUI Interface for learning a new thought
 *
 * This script provides the interface between the handler and
 * the Smarty template engine. The handler instantiates a
 * presentation object and then calls its methods. These
 * methods call Smarty templates that create and display HTML.
 *  
 * @author			$Author: richtl $
 * @version		$Revision: 973 $
 * @copyright	&copy; 2001, 2003, ars Cognita Inc., all rights reserved
 *
 * @package		mindmeld
 * @subpackage	learn
 *
 * @internal
 */
/********************************************************************************

/**
* Include general purpose data structures for the Smarty interface
*/
require_once ( $MM_GLOBALS['home'] . "interfaces/smarty/include/interface_smarty.inc"  );

/**
* Include high-level mindmeld classes
*/
require_once( $MM_GLOBALS['home'] . "include/mind.inc.php" );

/**
* class learnPresentation
*
* The learnPresentation class provides GUI methods for adding new answers to 
* the database.
*/
class learnPresentation {
		
	/*
	* @var int unique assigned ID
	*/
	var $thoughtId;
	
	/*
	* @var string Short summary of thought
	*/
	var $thoughtSummary;
	
	/*
	* @var string Detailed answer
	*/
	var $thoughtDetail;
	
	/**
	* Class constructor
	*
	* Doesn't do anything at the moment.
	*/
	function learnPresentation() {
	}

	/**
	* Build and display form to enter new answer
	*
	* This is the user interface for adding a new thought to the system. It presents
	* a form to the user, requesting a summary and detail for the thought.
	*
	* @param	bool	$checkRequired			Check for required fields (TRUE/FALSE)
	*/
	function getContent( $checkRequired = FALSE ) {
		
		global $MM_GLOBALS;
		
		// Move this to a seperate plugin environment
		$site = $MM_GLOBALS['site'];
		
		$body = '';
		
		// Enable or disable the wysiwyg editor plugin
		//$plugin_initBody = 'onload="initEditor()"';
		$mm = new mindmeld();
		$enableEditor = $mm->readParam( 'wysiwyg_editor' );
		if( $enableEditor ) $body .=  'onload="HTMLArea.replaceAll()"';

		$plugin_initBody = $body;
		
		// Create and display the HTML template.
		
		$template = new mmHtmlTemplate;	
		$template->assign( "editorEnabled", $enableEditor );
		$template->assign( "thoughtSummary", $this->thoughtSummary );
		$template->assign( "thoughtDetail", postfilterHtml( $this->thoughtDetail ) );
		$template->assign_by_ref( "checkRequired", $checkRequired );
		// Embed plugins
		$template->assign_by_ref( "plugin_initBody", $plugin_initBody );
		
		$template->display( "learn_getContent.tpl" );
	}
	
	/**
	* Build and display new answer preview
	*
	* This is the user interface for previewing a thought that is being added or
	* modified. It shows the author what the thought will look like to the
	* typical user.
	*
	* The detail is filtered before being displayed as follows:
	*	HTML: If the detail contains <HTML> and </HTML> tags, it will be displayed
	*			as HTML.
	*	PRE: If the detail begins with <PRE> and ends with </PRE>, it will be
	*			displayed as preformatted HTML.
	*	Otherwise, the detail will displayed as freeform text, and newlines will
	*			be added as appropriate.
	*/
	function previewResults() {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->assign( "thoughtSummary", $this->thoughtSummary );
		$template->assign( "thoughtDetail", postfilterHtml( $this->thoughtDetail, TRUE, "AUTO" ) );
		$template->assign( "thoughtId", "(unassigned)" );
		$template->display( "ask_showAnswer.tpl" );
	}

	/**
	* Build and display new answer
	*
	* This is the user interface for displaying a thought after it has been created
	* or modified. It should thank the user, offer appropriate maintenance
	* instructions, and let them start over.
	*/
	function displayThought() {

		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->assign( "thoughtId", $this->thoughtId );
		$template->assign( "thoughtSummary", $this->thoughtSummary );
		$template->assign_by_ref( "thoughtDetail", postfilterHtml( $this->thoughtDetail, TRUE, "AUTO" ) );
		$template->display( "learn_showNewThought.tpl" );
	}
}

/**
* class mmLearn
*
* The learnPresentation class provides logic methods for adding new answers to 
* the database.
*/
class mmLearn extends learnPresentation {
	
	/**
	* Add a new answer to the database
	*
	* This is the logic that actually adds the new thought to the mindmeld
	* system.
	*/
	function addThought() {
		// Create a new mind object and add the thought.
		$mind = new mind();
		
		// TODO: Catch failure if it occurs
		$this->thoughtId = $mind->learnThought( $this->thoughtSummary, $this->thoughtDetail );
		
		return $mind;
	}
}
?>
