<?PHP
// Copyright (c) 2003 ars Cognita, Inc., all rights reserved
/********************************************************************************
	
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * HTML GUI to manage content
 *  
 * @author		$Author: $
 * @version		$Revision: 1213 $
 *
 * @package interfaces
 * @subpackage smarty
 */

/**
 * Smarty general interface module
 */
require_once ( $MM_GLOBALS['home'] . "interfaces/smarty/include/interface_smarty.inc"  );

/**
 * High-level Mindmeld module
 */
require_once( $MM_GLOBALS['home'] . "include/mind.inc.php" );

/**
* class managePresentation
*
* The managePresentatoin class provides presentation methods for building web
* interfaces for managing content.
*/
class managePresentation {
	
	/**
	 * @var string Submitted query string
 	 */
	var $theQuestion;
	
	/**
	 * @var integer Unique ID for this query
	 */
	var $queryId;
	
	/**
	 * @var integer Unique ID for the selected thought
	 */
	var $thoughtId;
	
	/**
	 * @var string Short summary of thought
	 */
	var $thoughtSummary;
	
	/**
	 * @var string Detailed answer
	 */
	var $thoughtDetail;
	
	/**
	 * @var object thought
	 */
	var $thought;
	
	/**
	 * @var array Array of actions
	 */
	var $actionArray;
	
	/**
	* Class constructor
	*
	* Doesn't do anything at the moment.
	*/
	function mmManage() {
	}
	
	/**
	* Build and display the interface for searching for content
	*
	* Part of the user interface for searching for content. This function gets the 
	* actual query text from the user.
	*/
	function showIndex() {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->assign_by_ref( "theQuestion", $this->theQuestion );
		$template->assign_by_ref( "thoughtId", $this->thoughtId );
		$template->assign_by_ref( "theQuestion", $this->theQuestion );
		$template->display( "manage_manageIndex.tpl" );
	}

	/**
	* Build and display the interface to display search results list
	*
	* Part of the user interface for searching for content. This function lists the 
	* possible matches for the user to choose from.
	*
	* @param	array	$results		Array of summaries that match the user's query
	*/
	function showResults( $results = NULL ) {
		
		global $MM_GLOBALS;
		
		$urlSelf = $MM_GLOBALS['url_manage'];		
		strrpos( $urlSelf, '?' ) ? ( $link = $urlSelf."&" ) : ( $link = $urlSelf."?" );
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->assign_by_ref( "theQuestion", $this->theQuestion );
		$template->assign_by_ref( "queryId", $this->queryId );
		$template->assign_by_ref( "baseLink", $link );
		$template->assign_by_ref( "answers", $results );
		$template->display( "manage_showResults.tpl" );
	}
	
	/**
	* Build and display the interface for modifying an existing answer
	*
	* This is the user interface for modifying an existing thought in the system.
	* The process works thusly: If a thoughtID, summary, and detail are provided
	* as arguments, the user is presented with a form similar to the "create thought"
	* form, but with the requested thought filled in and ready for editing. If no
	* thoughtID is passed, the user is presented with a form that asks for one, and
	* loads it, and then allows editing. Finally, if only a thoughtID is passed, the
	* thought is loaded and the user can edit.
	*/
	function updateContent() {
		
		global $MM_GLOBALS;
		
		// Move this to a seperate plugin environment
		$site = $MM_GLOBALS['site'];
		
		// Create and display the HTML template.
		// NOTE: Entering an ID and return without pressing the submit
		// 		button doesn't send the value of submit.
		
		$body = '';
		
		// Enable or disable the wysiwyg editor plugin
		//$plugin_initBody = 'onload="initEditor()"';
		$mm = new mindmeld();
		$enableEditor = $mm->readParam( 'wysiwyg_editor' );
		if( $enableEditor ) $body .=  'onload="HTMLArea.replaceAll()"';

		$plugin_initBody = $body;
		
		$template = new mmHtmlTemplate;	
		$template->assign( "editorEnabled", $enableEditor );
		$detail = postfilterHtml( $this->thoughtDetail );
		$template->assign_by_ref( "thoughtId", $this->thoughtId );
		$template->assign_by_ref( "thoughtSummary", $this->thoughtSummary );
		if ( $detail ) $template->assign_by_ref( "thoughtDetail", $detail );
		// Embed plugins
		$template->assign_by_ref( "plugin_initBody", $plugin_initBody );
		
		$template->display( "manage_getUpdate.tpl" );
	}
	
	/**
	* Build and display the interface for previewing an answer
	*
	* This is the user interface for previewing a thought that is being added or
	* modified. It shows the author what the thought will look like to the
	* typical user.
	*
	* The detail is filtered before being displayed as follows:
	*	HTML: If the detail contains <HTML> and </HTML> tags, it will be displayed
	*			as HTML.
	*	PRE: If the detail begins with <PRE> and ends with </PRE>, it will be
	*			displayed as preformatted HTML.
	*	Otherwise, the detail will displayed as freeform text, and newlines will
	*			be added as appropriate.
	*/
	function previewUpdate() {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$htmlObj = parseHtml( $this->thought->getDetail() );
		
		if( $this->thought->displayRaw ) {	// Display headers and unfiltered HTML
			$detail = $htmlObj->body;
			$this->showHeader( $this->thought );
		} else {	// Display normal (filtered) HTML and no headers
			$detail = postfilterHtml( $this->thoughtDetail, TRUE, "AUTO" );
		}
			
		$template->assign_by_ref( "thoughtId", $this->thoughtId );
		$template->assign_by_ref( "thoughtSummary", $this->thoughtSummary );
		$template->assign( "thoughtPreview", $detail );
		$template->assign( "thoughtDetail", postfilterHtml( $this->thoughtDetail ) );
		$template->assign( "preview", "TRUE" );
		$template->display( "manage_getUpdate.tpl" );
	}
	
	/**
	* Build and display the interface for displaying a modified answer
	*
	* This is the user interface for displaying a thought after it has been modified.
	* It should thank the user, offer appropriate maintenance instructions, and let 
	* them start over.
	*/
	function updateAndDisplayThought() {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$htmlObj = parseHtml( $this->thought->getDetail() );
		
		if( $this->thought->displayRaw ) {	// Display headers and unfiltered HTML
			$detail = $htmlObj->body;
			$this->showHeader( $this->thought );
		} else {	// Display normal (filtered) HTML and no headers
			$detail = postfilterHtml( $this->thoughtDetail, TRUE, "AUTO" );
		}
		
		$template->assign_by_ref( "thoughtId", $this->thoughtId );
		$template->assign_by_ref( "thoughtSummary", $this->thoughtSummary );
		$template->assign_by_ref( "thoughtDetail", $detail );;
		$template->assign( "update", "TRUE" );		// Display as update
		$template->assign( "formSubmit", "Start Over" );
		$template->display( "manage_showThought.tpl" );
	}
	
	/**
	* Build and display the interface for the search header page
	*/
	function showQueryHeader() {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->display( "manage_queryHeader.tpl" );
	}
	
	/**
	* Build and display the interface for the results list header page
	*/
	function showResultsHeader() {
		
		// Create and display the HTML template.
		$template = new mmHtmlTemplate;	
		$template->display( "manage_resultsHeader.tpl" );
	}

	/**
	* Build and display the the HTML header
	*
	* Wrapper that calls the general mmsmarty_showHeader function from
	* interface_smarty.inc. This method takes a thought, checks its displayRaw
	* property to see if headers should be displayed, and if so, sends script
	* and style tags.
	*
	* @param object $thought thought object
	*/
	function showHeader( $thought ) {
		
		if( !$thought->displayRaw ) return NULL;
		
		$hdrObj = parseHtml( $thought->getDetail() );
		unset( $hdrObj->headerMetaTags );
		unset( $hdrObj->headerMiscTags );
				
		mmsmarty_showHeader( $hdrObj );
	}
}

/**
* class mmManage
*
* The mmManage class provides logic methods for building web interfaces for
* managing content.
*/
class mmManage extends managePresentation {
	
	/**
	* Fetch the search results list
	*
	* getResults fetches the result list that matches the user's query.
	*
	* @return	array	Array containing results list
	*/
	function getResults() {
		
		global $MM_GLOBALS;
		
		// Build the link list to display the possible matches.
		
		if ( $this->theQuestion != NULL || $this->thoughtId == NULL ) {		// Perform the search
			
			$mind = new mind();
			
			// If we don't have a queryID, then we need to run the query. Otherwise, just
			// refetch the answers.
			
			if ( !$this->queryId ) {
				$this->queryId = $mind->ask( stripslashes( $this->theQuestion ), $MM_GLOBALS['user_email'], "%" );
			}
			$results = $mind->answers( $this->queryId, '%', FALSE );	
			
		} else { 	// Fetch the thought directly from the ID
			
			$thought = new thought();
			$this->thoughtId = $thought->readThoughtQuick( $this->thoughtId, FALSE );
			
			if ( $this->thoughtId ) {
				$results[0][0] = $this->thoughtId;
				$results[0][1] = $thought->getSummary();
				$results[0][2] = 0;
				$results[0][3] = $thought->getStatus();
			}
		}	
		return $results;
	}
	
	/**
	* Get an answer by ID
	*
	* Fetch the current thought by ID.
	*
	* @return thought Requested answer
	*/
	function fetchThought() {
		
		$thought = NULL;
		
		// Make sure we have a thoughtId, then use it to load the thought.
		if( $this->thoughtId ) {
			
			if( $this->thoughtSummary == "" ) $this->thoughtSummary = NULL;
			if( $this->thoughtDetail == "" ) $this->thoughtDetail = NULL;
			
			if( $this->thoughtSummary == NULL || $this->thoughtDetail == NULL ) {
				
				// TODO: Need to handle failure elegantly
				// TODO: extract from presentation section
				
				$thought = new thought();
				$thought->readThoughtFull( $this->thoughtId, FALSE );
				$this->thought = $thought;
				
				// Deprecate!
				$this->thoughtSummary = $thought->getSummary();
				$this->thoughtDetail = $thought->getDetail();			
			}
		}
		return $thought;
	}
	
	/**
	* Update an answer by ID
	*
	* Update the current thought by ID.
	*
	* @return mind 	Mind object
	*/
	function updateThought() {
		
		// Update the thought in the database.
		// TODO: Catch failure if it occurs
		
		$mind = new mind();
		
		$mind->updateLearnedThought( $this->thoughtId, $this->thoughtSummary, $this->thoughtDetail );
		
		return $mind;
	}
	
	/**
	* Update all answers in the actions array
	*
	* Update the all thoughts in the actions array
	*/
	function commitChanges() {
		
		// Commit the actions (changes)
		foreach ( $this->actionArray as $thoughtId => $action ) {
			
			$thought = new thought();
			$thought->setThoughtId( $thoughtId );
			
			// Fetch the current status. Only need to update if changed.
			$thought->readThoughtQuick( $thoughtId );
			$currStatus = $thought->getStatus();
			$thought->logMsg( "Thought $thougthId status was '$currStatus', is '$action'", MM_ALL );
			
			if( $thought->getStatus() != $action ) {
				$thought->logMsg( "Updating status on thought $thoughtId to $action", MM_INFO );
				$thought->setStatus( $action );
				$thought->updateThought();
			}
		}
	}
}
?>
