#!/usr/bin/php -q
<?PHP
// Copyright (c) 2002-2003 ars Cognita, Inc., all rights reserved
/********************************************************************************
	
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * Dream-time utility to clear the SQL cache.
 *
 * Script to perform all dreamstate activities. This script should be run on 
 * a periodic basis (i.e., nightly). Currently the script does the following:
 *	
 *	- update_neural_counts.php: update the count of the number of neurons linked
 *		to a given symbol.
 *	- noisefilter.php: Filter out noise-symbols.
 *	- negatively reinforces tsneurons attached to unused decisions
 *	- Inserts query_history information into the query history table.
 *	- deletes stale stimuli, decisions, and queries from the database
 *	- handles adodb cache garbage collection
 *	- temporarily ignores feetback management
 *  
 * @author		$Author: richtl $
 * @version		$Revision: 1022 $
 *
 * @package dream
 */

/*
* Locate all relevant mindmeld.config files. This is used primarily if we have
* more than one acweb directory pointed at more than one database.
*
* Read mindmeld/instance.ini (if it exists) for locations of all acweb directories.
* instance.ini should only contain ONE ENTRY FOR A GIVEN DATABASE!
*
* If no entries are found in instance.ini or there is no instance.ini, use the
* parent directory (of mindmeld/php) to load the mindmeld.config file.
*/
if( file_exists( "../instance.ini" ) ) {
	$instance = parse_ini_file( "../instance.ini" );
}
if( !$instance ) $instance[] = ".."; // Add the mindmeld home directory

/**
* Configuration handler: load the mindmeld/mindmeld.config file first,
* then load each additional instance (as necessary).
*/
include( '../include/header.inc' );
$config = new mmConfiguration( FALSE );
$config->loadConfiguration( '../mindmeld.config' );

// Loop through each instance, loading its mindmeld.config file and then dreaming.
foreach( $instance as $instPath ) {
	
	// Get the full path to the mindmeld.config file from the instance array and load
	// the configuration. Then load the default config and set up the MM_GLOBALS array.
	$configPath = trim( $instPath );
	$configPath = preg_replace( '/\/$/', '', $configPath ) . "/mindmeld.config";
	$config->loadConfiguration( $configPath, TRUE );
	$config->loadDefaultConfiguration();	
	$config->setGlobals();
	
	/**
	* Include the main mindmeld library
	* We have to load it late, after the adodb path is configured.
	*/
	require_once( $MM_GLOBALS['home'] . "include/utilities.inc" );
	
	$today = date( "M-d-Y H:i:s" );
	$dreamInfo = "Dreaming for database {$MM_GLOBALS['dbName']} on server {$MM_GLOBALS['dbHost']}";
	
	print "<H1>Dream Report for $today</H1>\n";
	print "<P>$dreamInfo</P>";
	print "<H3>Starting to dream...</H3><HR/>\n";
	
	$mm = new mindmeld();
	
	$event = new mmEvent;
	$event->dream( 'start dream', $dreamInfo  );
		
	// PLUGIN HOOK: Manipulate parsed symbols list
	$mm = $mm->pluginMgr->executePlugins( 'dream', $mm );
	
	$event->dream( 'stop dream', $dreamInfo );
	print "<HR/><H3>Dreaming complete.</H3>\n";
}
?>
