<?PHP
// Copyright (c) 2003 ars Cognita, Inc., all rights reserved
/********************************************************************************
	
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; version 2 of the License.
   
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*******************************************************************************/
/**
 * Core filter plugins for Mindmeld
 *
 * Provides core data filter plugins for mindmeld
 *  
 * @author		$Author: $
 * @version		$Revision: 1213 $
 *
 * @package plugins
 * @subpackage core
 */

/**
 * Plugin to dream
 */
class p50_dream_resolveQueries extends plugin {
	
    /**
    * Dream-time utility to resolve all queries, manage statuses, clean up.
    *
	* @param object $dream Dream object
	*/
	function dream( $dream ) {
		
		$queryAge = $dream->readParam( 'stale_query_age' );
		
		// Connect to the database and execute the query
		$dbconn = $dream->dbconn;
		
		// These status queries must execute in the specified order
		
		// Set queries containing "unsure" decisions to "open pending"
		$attrib = array( 	'%QSTATUS%' => 'OPEN_PENDING',
							'%DSTATUS%' => 'UNSURE',
							'%DELAY%' => $queryAge );
		$record = $dream->getAction( 'update query states by decision', $attrib );
		
		// Set queries containing "rejected" decisions to "closed no"
		$attrib = array( 	'%QSTATUS%' => 'CLOSED_NO',
							'%DSTATUS%' => 'REJECTED',
							'%DELAY%' => $queryAge );
		$record = $dream->getAction( 'update query states by decision', $attrib );
		
		// Set queries containing "accepted" decisions to "closed yes"
		$attrib = array( 	'%QSTATUS%' => 'CLOSED_YES',
							'%DSTATUS%' => 'ACCEPTED',
							'%DELAY%' => $queryAge );
		$record = $dream->getAction( 'update query states by decision', $attrib );
		
		// Abort stale "new" queries
		$attrib = array( 	'%OLDSTATUS%' => 'NEW',
							'%NEWSTATUS%' => 'CLOSED_ABORTED',
							'%DELAY%' => $queryAge );
		$record = $dream->getAction( 'update query states by query', $attrib );
		
		// Close stale "open" queries
		$attrib = array( 	'%OLDSTATUS%' => 'OPEN',
							'%NEWSTATUS%' => 'CLOSED',
							'%DELAY%' => $queryAge );
		$record = $dream->getAction( 'update query states by query', $attrib );
		
		// Remove null or empty decisions.
		// This MUST be done before inserting the query history.
		$record = $dream->getAction( 'clear empty decisions' );
		
		// Move all closed queries and decisions to query_history
		$record = $dream->getAction( 'insert query history' );
		
		// Delete all closed queries and decisions, and stimuli
		$record = $dream->getAction( 'clear stimuli' );
		
		$record = $dream->getAction( 'clear decisions' );
		
		$record = $dbconn->Execute( "DELETE FROM Query WHERE status LIKE 'CLOSED%'" );
		
		$event = new mmEvent;
		$event->dream( get_class( $this ) );
		return NULL;
	}
}
	
 /**
* Dream-time utility to follow up on all queries with a status of "PENDING."
*/
class p50_dream_followupPending extends plugin {
	
    /**
    * Dream-time utility to follow up on all queries with a status of "PENDING."
    *
	* @param object $dream Dream object
	*/
	function dream( $dream ) {
		
		$queryAge = $dream->readParam( 'stale_query_age' );
		
		// Connect to the database and execute the query
		$dbconn = $dream->dbconn;
		
		// These status queries must execute in the specified order
		
		// Change pending status to closed.
		$attrib = array( 	'%OLDSTATUS%' => 'OPEN_PENDING',
							'%NEWSTATUS%' => 'CLOSED_PENDING',
							'%DELAY%' => $queryAge );
		$record = $dream->getAction( 'update query states by query', $attrib );
		
		$event = new mmEvent;
		$event->dream( get_class( $this ) );
		return NULL;
	}
}

/**
* Dream-time utility to negatively reinforce neurons that were rejected in 
* searches
*/
class p50_dream_reinforceNeurons extends plugin {
	
    /**
    * Dream-time utility to negatively reinforce neurons that were rejected in 
 	* searches.
    *
	* @param object $dream Dream object
	*/
	function dream( $dream ) {
		
		/**
		* High-level mind library
		*/
		include_once( $dream->mmconfig['home'] . 'include/mind.inc.php');
		
		$mind = new mind();
		$mind->reinforceMemory( NULL, NULL, "NEGATIVE" );
		
		$event = new mmEvent;
		$event->dream( get_class( $this ) );
		return NULL;
	}
}
?>